from pandas import DataFrame
from sklearn.base import ClassifierMixin
from dataclasses import dataclass
from typing import List, Dict, Any, Optional, Union


@dataclass(frozen=True)
class cat2cat_data:
    """The dataclass to represent a data argument used in cat2cat procedure

    Args:
        old (DataFrame): older time point in a panel, has to have all columns set in the rest of arguments.
        new (DataFrame): newer time point in a panel, has to have all columns set in the rest of arguments.
        cat_var_old (str): name of the categorical variable in the older time point.
        cat_var_new (str): name of the categorical variable in the newer time point.
        time_var (str): name of the time variable.
        id_var (Optional[str]): name of the unique identifier variable - if this is specified then for subjects observe in both periods the direct mapping is applied.
        multiplier_var (Optional[str]): name of the multiplier variable - number of replication needed to reproduce the population.
    """

    old: DataFrame
    new: DataFrame
    cat_var_old: str
    cat_var_new: str
    time_var: str
    id_var: Optional[str] = None
    multiplier_var: Optional[str] = None

    def __post_init__(self):
        assert isinstance(self.old, DataFrame), "old has to be a pandas.DataFrame"
        assert isinstance(self.new, DataFrame), "new has to be a pandas.DataFrame"
        assert isinstance(self.cat_var_old, str) and (
            self.cat_var_old in self.old.columns
        ), "cat_var_old has to be a str and the old DataFrame column"
        assert isinstance(self.cat_var_new, str) and (
            self.cat_var_new in self.new.columns
        ), "cat_var_new has to be a str and the new DataFrame column"
        assert isinstance(self.time_var, str) and (
            (self.time_var in self.new.columns) and (self.time_var in self.old.columns)
        ), "time_var has to be a str and in the both (old and new) DataFrames"
        assert (len(self.old[self.time_var].unique()) == 1) and (
            len(self.new[self.time_var].unique()) == 1
        ), "time_var has to have only the one period in each DataFrame (old and new)"
        assert (
            isinstance(self.id_var, str)
            and (
                (self.id_var in self.old.columns) and (self.id_var in self.new.columns)
            )
        ) or (
            self.id_var is None
        ), "id_var has to be a str and in the both (old and new) DataFrames, or None"
        assert (
            isinstance(self.multiplier_var, str)
            and (
                (self.multiplier_var in self.new.columns)
                or (self.multiplier_var in self.old.columns)
            )
        ) or (
            self.multiplier_var == None
        ), "multiplier_var has to be a str and in one of (old and new) DataFrames, or None"


@dataclass(frozen=True)
class cat2cat_mappings:
    """The dataclass to represent a mappings argument used in cat2cat procedure

    Args:
        trans (DataFrame): mapping (transition) table (with 2 columns, old and new encoding) - all categories for cat_var in old and new datasets have to be included.
        diretion (str): "backward" or "forward"
        freqs (Optional[str]): If It is not provided then is assessed automatically.
                               Artificial counts for each variable level in the base period.
                               It is optional nevertheless will be often needed, as gives more control.

    Note:
        The mapping (transition) table should to have a candidate for each category from the targeted for an update period.
        The observation from targeted for an updated period without a matched category from base period is removed.
    """

    trans: DataFrame
    direction: str
    freqs: Union[Dict[Any, int], None] = None

    def __post_init__(self):
        assert isinstance(self.trans, DataFrame), "trans has to be a pandas.DataFrame"
        assert self.trans.shape[1] == 2, "trans has to have two columns"
        assert isinstance(self.direction, str), "direction has to be a str"
        assert self.direction in [
            "forward",
            "backward",
        ], "direction has to be one of 'forward' or 'backward'"
        assert (self.freqs == None) or isinstance(
            self.freqs, dict
        ), "freqs has to be a pandas.DataFrame with 2 columns, or None"


@dataclass(frozen=True)
class cat2cat_ml:
    """The dataclass to represent a ml argument used in cat2cat procedure

    Args:
        data (DataFrame): dataset with features and the `cat_var`.
        cat_var (str): the dependent variable name.
        features (List[str]): list of features names where all have to be numeric or logical
        models (List[ClassifierMixin]): scikit-learn instances (classes inherit from ClassifierMixin) like,
                                        RandomForestClassifier() or LinearDiscriminantAnalysis() instances.
    """

    data: DataFrame
    cat_var: str
    features: List[str]
    models: List[ClassifierMixin]

    def __post_init__(self):
        assert isinstance(self.data, DataFrame), "data has to be a pandas.DataFrame"
        assert isinstance(self.cat_var, str) and (
            self.cat_var in self.data.columns
        ), "cat_var has to be a str and a data argument column"
        assert isinstance(self.features, list) and all(
            [e in self.data.columns for e in self.features]
        ), "features has to be a list and each have to be a column in the data argument."
        assert isinstance(self.models, list) and (
            len(self.models) > 0
        ), "models has to be a list of length at least 1."
        assert all(
            [issubclass(type(e), ClassifierMixin) for e in self.models]
        ), "models arg elements have to be subclass of ClassifierMixin each"
        assert all(
            [hasattr(e, "fit") for e in self.models]
        ), "each model has to have the fit method"
        assert all(
            [hasattr(e, "predict_proba") for e in self.models]
        ), "each model has to have the (multi-label) predict_proba method"



from importlib_resources import files, as_file
from pandas import read_pickle
import cat2cat.data

__all__ = ["load_trans", "load_occup", "load_verticals"]


def get_file_path(file):
    source = files(cat2cat.data).joinpath(file)
    return source


def load_verticals():
    """load trans dataset
    trans dataset containing mappings (transitions) between old (2008) and new (2010) occupational codes

    Returns:
        pandas.DataFrame: trans dataset
    """
    sour = get_file_path("verticals.pkl")
    with as_file(sour) as fil:
        return read_pickle(fil)


def load_trans():
    """load trans dataset
    trans dataset containing mappings (transitions) between old (2008) and new (2010) occupational codes

    Returns:
        pandas.DataFrame: trans dataset
    """
    sour = get_file_path("trans.pkl")
    with as_file(sour) as fil:
        return read_pickle(fil)


def load_occup(small=False):
    """load occup dataset

    occup dataset is an example of unbalance panel dataset.
    This is a simulated data although there are applied a real world characteristics from national statistical office survey.
    The original survey is anonymous and take place every two years.
    It is presenting a characteristics from randomly selected company and then using k step procedure employees are chosen.

    Args:
        small (bool): if to use a shrinked version of dataset

    Returns:
        pandas.DataFrame: occup dataset
    """
    sour = get_file_path("occup_small.pkl" if small else "occup.pkl")
    with as_file(sour) as fil:
        return read_pickle(fil)



from pandas import DataFrame, concat
from numpy import arange, repeat, setdiff1d, in1d, intersect1d

from cat2cat.mappings import get_mappings, get_freqs, cat_apply_freq
from cat2cat.dataclass import cat2cat_data, cat2cat_mappings, cat2cat_ml
from cat2cat.cat2cat_utils import dummy_c2c

from typing import Optional, Any, Dict

__all__ = ["cat2cat"]


def cat2cat(
    data: cat2cat_data, mappings: cat2cat_mappings, ml: Optional[cat2cat_ml] = None
):
    """Automatic mapping in a panel dataset - cat2cat procedure

    Args:
        data (cat2cat_data): dataclass with data related arguments.
            Please check out the `cat2cat.dataclass.cat2cat_data` for more information.
        mappings (cat2cat_mappings): dataclass with mappings related arguments.
            Please check out the `cat2cat.dataclass.cat2cat_mappings` for more information.
        ml (Optional[cat2cat_ml]): dataclass with ml related arguments.
            Please check out the `cat2cat.dataclass.cat2cat_ml` for more information.

    Returns:
        dict: with 2 DataFrames, old and new.
        There will be added additional columns like index_c2c, g_new_c2c, wei_freq_c2c, rep_c2c, wei_(ml method name)_c2c.
        Additional columns will be informative only for a one DataFrame as we always make the changes to one direction.

    Note:
        1. Without ml section only simple frequencies are assessed.
        When ml model is broken then weights from simple frequencies are taken.
        `knn` method is recommended for smaller datasets.

        2. Please be sure that the categorical variable is of the same type in all cases.
        `mappings.trans` arg columns and the `data.cat_var` column have to be of the same type.
        When ml part applied `ml.cat_var` has to have the same type too.

    >>> from cat2cat import cat2cat
    >>> from cat2cat.dataclass import cat2cat_data, cat2cat_mappings, cat2cat_ml
    >>> from sklearn.ensemble import RandomForestClassifier
    >>> from cat2cat.datasets import load_trans, load_occup
    >>> trans = load_trans()
    >>> occup = load_occup()
    >>> o_old = occup.loc[occup.year == 2008, :].copy()
    >>> o_new = occup.loc[occup.year == 2010, :].copy()
    >>> data = cat2cat_data(old = o_old, new = o_new, cat_var_old = "code",
    ...                     cat_var_new = "code", time_var = "year")
    >>> mappings = cat2cat_mappings(trans = trans, direction = "forward")
    >>> cat2cat(data = data, mappings = mappings)
    {...

    """
    assert isinstance(data, cat2cat_data), "data arg has to be cat2cat_data instance"
    assert isinstance(
        mappings, cat2cat_mappings
    ), "mappings arg has to be cat2cat_mappings instance"
    assert (ml is None) or isinstance(
        ml, cat2cat_ml
    ), "ml arg has to be cat2cat_ml instance"

    mapps = get_mappings(mappings.trans)

    if mappings.direction == "forward":
        target_name = "new"
        base_name = "old"
    elif mappings.direction == "backward":
        target_name = "old"
        base_name = "new"

    cat_var_base = getattr(data, "cat_var_" + base_name)
    cat_var_target = getattr(data, "cat_var_" + target_name)
    base_df = getattr(data, base_name).copy()
    target_df = getattr(data, target_name).copy()
    mid_df = DataFrame()
    mapp = mapps["to_" + base_name]

    is_direct = not data.id_var is None
    if is_direct:
        id_inner = intersect1d(base_df[data.id_var], target_df[data.id_var])
        id_outer = setdiff1d(target_df[data.id_var], base_df[data.id_var])
        mid_df = dummy_c2c(
            target_df.loc[target_df[data.id_var].isin(id_inner), :].copy(),
            cat_var_base,
        )
        target_df = target_df.loc[target_df[data.id_var].isin(id_outer)].copy()

        tos = data.old.loc[:, [data.id_var, data.cat_var_old]].merge(
            data.new.loc[:, [data.id_var, data.cat_var_new]], on=data.id_var
        )
        tos.columns = ["id", "cat_old", "cat_new"]
        tos_dict = dict(zip(tos["id"], tos["cat_" + base_name]))
        mid_df["g_new_c2c"] = [tos_dict.get(e) for e in mid_df[data.id_var]]

    # frequencies
    freqs = _resolve_frequencies(
        base_df, cat_var_base, mappings.freqs, data.multiplier_var
    )
    # frequencies per category
    mapp_f = cat_apply_freq(mapp, freqs)

    # mappings and frequencies per obs
    a_mapp = [mapp.get(e, []) for e in target_df[cat_var_target]]
    a_mapp_f = [mapp_f.get(e, []) for e in target_df[cat_var_target]]
    lens = [len(e) for e in a_mapp]
    nrow_target = target_df.shape[0]

    # target_df
    target_df = target_df.assign(index_c2c=arange(nrow_target))
    target_df = target_df.iloc[repeat(arange(nrow_target), lens), :]
    target_df = target_df.assign(
        g_new_c2c=[e for l in a_mapp for e in l],
        rep_c2c=repeat(lens, lens),
    )
    target_df = target_df.assign(
        wei_naive_c2c=1 / target_df.rep_c2c,
        wei_freq_c2c=[e for l in a_mapp_f for e in l],
    )
    target_df = target_df.reset_index(drop=True)

    # base_df
    base_df = dummy_c2c(base_df, cat_var_base)

    # ML
    if ml is not None:
        for m in ml.models:
            ml_name = type(m).__name__
            ml_colname = "wei_" + ml_name + "_c2c"
            target_df[ml_colname] = target_df["wei_freq_c2c"]
            base_df[ml_colname] = 1
            if is_direct:
                mid_df[ml_colname] = 1

        _cat2cat_ml(ml, mapp, target_df, cat_var_target)

    # Final
    res = dict()
    res[target_name] = concat([target_df, mid_df])
    res[base_name] = base_df

    return res


def _cat2cat_ml(ml, mapp, target_df, cat_var_target):
    """cat2cat ml optional part"""
    for target_cat in list(mapp.keys()):
        base_cats = mapp[target_cat]
        ml_cat_var = ml.data[ml.cat_var]

        if (not any(in1d(base_cats, ml_cat_var.unique()))) or (len(base_cats) == 1):
            continue

        target_cat_index = in1d(target_df[cat_var_target].values, target_cat)
        ml_cat_index = in1d(ml.data[ml.cat_var].values, base_cats)

        data_ml_train = ml.data.loc[ml_cat_index, :]
        data_ml_target = target_df.loc[target_cat_index, :]

        target_cats = data_ml_target["g_new_c2c"]
        data_ml_target_uniq = data_ml_target.drop_duplicates(
            subset=["index_c2c"] + ml.features
        )
        index_c2c = data_ml_target_uniq["index_c2c"].values

        for m in ml.models:
            ml_name = type(m).__name__
            ml_colname = "wei_" + ml_name + "_c2c"

            try:
                m.fit(X=data_ml_train.loc[:, ml.features], y=data_ml_train[ml.cat_var])

                X_test = data_ml_target_uniq.loc[:, ml.features]
                preds = m.predict_proba(X=X_test)

                preds_df = DataFrame(preds)
                preds_df.columns = m.classes_
                preds_df.loc[:, setdiff1d(target_cats.unique(), m.classes_)] = 0
                preds_df["index_c2c"] = index_c2c
                preds_df_melt = preds_df.melt(id_vars="index_c2c", var_name="g_new_c2c")
                merge_on = ["index_c2c", "g_new_c2c"]
                p_order = target_df.loc[target_cat_index, merge_on].merge(
                    preds_df_melt, on=merge_on, how="left"
                )
                target_df.loc[target_cat_index, ml_colname] = p_order["value"].values
            except:
                None


def _resolve_frequencies(
    base_df: DataFrame,
    cat_var_base: str,
    user_freqs: Optional[Dict[Any, int]],
    multiplier_var: Optional[str],
):
    """Resolve the frequencies"""
    freqs: Dict[Any, int] = dict()
    if user_freqs == None:
        if "wei_freq_c2c" in base_df.columns:
            freqs = (
                base_df.groupby(cat_var_base)
                .apply(lambda x: sum(x["wei_freq_c2c"] * x.get(multiplier_var, 1)))
                .to_dict()
            )
        else:
            freqs = get_freqs(
                base_df[cat_var_base].values, base_df.get(multiplier_var, None)
            )
    else:
        freqs = user_freqs
    return freqs



from pandas import DataFrame
from numpy import ndarray, unique, repeat, array, round

from collections.abc import Iterable
from typing import Union, Optional, Any, List, Dict, Sequence


def get_mappings(x: Union[DataFrame, ndarray]) -> Dict[str, Dict[str, List[Any]]]:
    """Transforming a mapping table with mappings to two associative lists

    Transforming a transition table with mappings to two associative lists
    to rearrange the one classification encoding into another, an associative list that maps keys to values is used.
    More precisely, an association list is used which is a linked list in which each list element consists of a key and value or values.
    An association list where unique categories codes are keys and matching categories from next or previous time point are values.
    A transition table is used to build such associative lists.

    Args:
        x (pandas.DataFrame or numpy.ndarray): transition table with 2 columns where first column is assumed to be the older encoding.

    Returns:
        dict: with 2 dicts, `to_old` and `to_new`.

    >>> from cat2cat.mappings import get_mappings
    >>> from numpy import array
    >>> trans = array([
    ...   [1111, 111101], [1111, 111102], [1123, 111405],
    ...   [1212, 112006], [1212, 112008], [1212, 112090],
    ... ])
    >>> mappings = get_mappings(trans)
    >>> mappings["to_old"]
    {112006: [1212], 112008: [1212], 111405: [1123], 112090: [1212], 111101: [1111], 111102: [1111]}
    >>> mappings["to_new"]
    {1123: [111405], 1212: [112006, 112008, 112090], 1111: [111101, 111102]}
    """

    assert (len(x.shape) == 2) and (
        x.shape[1] == 2
    ), "x should have 2 dimensions and the second one is equal to 2 (columns)"

    if isinstance(x, DataFrame):
        ff = x.iloc[:, 0].values
        ss = x.iloc[:, 1].values
    elif isinstance(x, ndarray):
        ff = x[:, 0]
        ss = x[:, 1]
    else:
        raise (TypeError)

    from_old = set(ff)
    from_new = set(ss)

    to_old = dict()
    for e in from_new:
        try:
            idx = ss == e
            # sorted so results are stable
            to_old[e] = sorted(list(set(ff[idx])))
        except:
            to_old[e] = []

    to_new = dict()
    for e in from_old:
        try:
            idx = ff == e
            # sorted so results are stable
            to_new[e] = sorted(list(set(ss[idx])))
        except:
            to_new[e] = []

    return dict(to_old=to_old, to_new=to_new)


def get_freqs(
    x: Sequence[Any], multiplier: Optional[Sequence[int]] = None
) -> Dict[Any, int]:
    """
    Getting frequencies from a vector with an optional multiplier

    Args:
        x (Sequence): a list like, categorical variable to summarize.
        multiplier (Optional[Sequence]): a list like, how many times to repeat certain value, additional weights.
                                         Have the same length as the x argument. Defaults to None.

    Returns:
        dict: with unique values and their counts

    >>> get_freqs([1,1,1,2,1,2,2,11])
    {1: 4, 2: 3, 11: 1}
    """
    assert isinstance(x, Iterable), "x has to be a Iterable"
    assert (multiplier is None) or isinstance(
        multiplier, Iterable
    ), "multiplier has to be a Iterable"
    input: ndarray
    if multiplier is not None:
        input = repeat(x, multiplier)
    else:
        input = array(x)
    input_unique: tuple = unique(input, return_counts=True)
    res: dict = dict(zip(*input_unique))
    return res


def cat_apply_freq(to_x: Dict, freqs: Dict[Any, int]) -> Dict[str, List[float]]:
    """
    Applying frequencies to the object returned by the `get_mappings` function

    Args:
        to_x (dict): object returned by `get_mappings` function
        freqs (dict): object like the one returned by the `get_freqs` function

    Returns:
        dict: the same shape as to_x arg but the values are probabilities now

    Note:
        freqs arg keys and to_x arg values have to be of the same type

    >>> from cat2cat.mappings import get_mappings, get_freqs, cat_apply_freq
    >>> from cat2cat.datasets import load_trans, load_occup
    >>> mappings = get_mappings(load_trans())
    >>> occup = load_occup()
    >>> codes_new = occup.code[occup.year == 2010].map(str).values
    >>> freqs = get_freqs(codes_new)
    >>> mapp_new_p = cat_apply_freq(mappings["to_new"], freqs)
    >>> mappings["to_new"]['3481']
    ['441401', '441402', '441403', '441490']
    >>> mapp_new_p['3481']
    [0.0, 0.6, 0.0, 0.4]
    """
    assert isinstance(to_x, dict), "to_x has to be a dict"
    assert isinstance(freqs, dict), "freqs has to be dict"
    res = dict()
    for x in to_x:
        cs = [freqs.get(e, 1e-12) for e in to_x[x]]
        fs = round(array(cs) / sum(cs), 10)
        res[x] = list(fs)
    return res



from pandas import DataFrame
from numpy import arange, concatenate, ndarray
from typing import Optional, Callable, List


def prune_c2c(
    df: DataFrame,
    prune_fun: Callable[[ndarray], ndarray],
    wei_var: str = "wei_freq_c2c",
    index_var: str = "index_c2c",
    inplace: bool = False,
) -> DataFrame:
    """Pruning which could be useful after the mapping process

    Args:
        df (DataFrame): a specific period from the cat2cat function result.
        prune_fun (callable): a function to process a 1D-array of weights (float) and return a 1D-array of boolean of the same length.
                              The weighs will be reweighted automatically to still to sum to one per each original observation.
        wei_var (str): By default "wei_freq_c2c".
        index_var (str): By default "index_c2c".
        inplace (bool): Whether to perform the operation inplace. By default False.
    Returns:
        DataFrame: df argument with possibly reduced number of rows.
    Note:
        - non-zero prune_fun - lambda x: x > 0
        - highest1 prune_fun - lambda x: arange(len(x)) == argmax(x)
        - highest prune_fun - lambda x: x == max(x)

    >>> from cat2cat import cat2cat
    >>> from cat2cat.dataclass import cat2cat_data, cat2cat_mappings, cat2cat_ml
    >>> from sklearn.ensemble import RandomForestClassifier
    >>> from cat2cat.datasets import load_trans, load_occup
    >>> trans = load_trans()
    >>> occup = load_occup()
    >>> o_old = occup.loc[occup.year == 2008, :].copy()
    >>> o_new = occup.loc[occup.year == 2010, :].copy()
    >>> data_c2c = cat2cat_data(o_old, o_new, "code", "code", "year")
    >>> mappings_c2c = cat2cat_mappings(trans, "forward")
    >>> c2c = cat2cat(data_c2c, mappings_c2c)
    >>> #
    >>> # non-zero - lambda x: x > 0
    >>> # highest1 - lambda x: arange(len(x)) == argmax(x)
    >>> # highest - lambda x: x == max(x)
    >>> #
    >>> # non-zero
    >>> prune_c2c(c2c["old"], lambda x: x > 0)
              id        age    sex  edu        exp  ...  index_c2c  g_new_c2c  rep_c2c wei_naive_c2c  wei_freq_c2c
    ...
    """
    assert isinstance(index_var, str), "index argument has to be a str"
    assert isinstance(wei_var, str), "col argument has to be a str"
    assert isinstance(df, DataFrame) and (
        (index_var in df.columns) and (wei_var in df.columns)
    ), "df argument has to be a DataFrame with the index and col args columns"
    assert callable(prune_fun), "prune_fun argument has to be a callable"
    assert isinstance(inplace, bool), "inplace argument has to be a bool"

    df2 = df if inplace else df.copy()
    final_rows = (
        df2.groupby(index_var, sort=False)
        .apply(lambda x: prune_fun(x[wei_var].values))
        .values
    )
    df2 = df2.loc[concatenate(final_rows)]
    # reweight
    df2[wei_var] = (
        df2.groupby(index_var, sort=False)[wei_var].apply(lambda x: x / sum(x)).values
    )
    return df2


def dummy_c2c(
    df: DataFrame, cat_var: str, models: Optional[List] = None, inplace: bool = False
) -> DataFrame:
    """Add default cat2cat columns to a `data.frame`

    The function is useful to achive consitent columns across all panel periods,
    even for ones for which cat2cat procedure was not applied.

    Args:
        df (DataFrame): a specific period from the cat2cat function result.
        cat_car (str): name of categorial variable
        models (Optional[list]): an optional list of str, ml models applied (class name).
                                 By default turn off, equal None.
        inplace (bool): Whether to perform the operation inplace. By default False.

    Returns:
        DataFrame: df arg DataFrame but with additional columns connected with cat2cat procedure.
        The base added columns if not already exist: index_c2c, g_new_c2c, rep_c2c, wei_naive_c2c, wei_freq_c2c.
        Additionaly ml models connected columns like wei_MLNAME_c2c.
    """
    assert isinstance(cat_var, str), "cat_var argument has to be a str"
    assert isinstance(df, DataFrame) and (
        (cat_var in df.columns)
    ), "df argument has to be a DataFrame with the cat_var column"
    assert (models == None) or isinstance(
        models, list
    ), "models has to be None or list of str (ml models)"
    assert isinstance(inplace, bool), "inplace argument has to be a bool"

    df2 = df if inplace else df.copy()
    nrow_df = df2.shape[0]

    if "index_c2c" not in df2.columns:
        df2["index_c2c"] = arange(nrow_df)
    if "g_new_c2c" not in df2.columns:
        df2["g_new_c2c"] = df2[cat_var]
    if "rep_c2c" not in df2.columns:
        df2["rep_c2c"] = 1
    if "wei_naive_c2c" not in df2.columns:
        df2["wei_naive_c2c"] = 1
    if "wei_freq_c2c" not in df2.columns:
        df2["wei_freq_c2c"] = 1

    if models is not None:
        for m in models:
            ml_col = "wei_" + m + "_c2c"
            if ml_col not in df2.columns:
                df2[ml_col] = 1

    return df2



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['cat2cat', 'cat2cat.data']

package_data = \
{'': ['*']}

install_requires = \
['importlib-resources>=5.9.0,<6.0.0',
 'numpy>=1.23.1,<2.0.0',
 'pandas>=1.4.3,<2.0.0',
 'scikit-learn>=1.1.2,<2.0.0',
 'scipy>=1.9.0,<2.0.0']

setup_kwargs = {
    'name': 'cat2cat',
    'version': '0.1.3',
    'description': 'Unifying an inconsistently coded categorical variable in a panel/longtitudal dataset.',
    'long_description': '# cat2cat \n<a href=\'https://github.com/polkas/py-cat2cat\'><img src=\'https://raw.githubusercontent.com/Polkas/cat2cat/master/man/figures/cat2cat_logo.png\' align="right" width="200px" style="margin:5px;"/></a>\n[![Build Status](https://github.com/polkas/py-cat2cat/workflows/ci/badge.svg)](https://github.com/polkas/py-cat2cat/actions)\n[![codecov](https://codecov.io/gh/Polkas/py-cat2cat/branch/main/graph/badge.svg)](https://codecov.io/gh/Polkas/py-cat2cat)\n[![pypi](https://img.shields.io/pypi/v/cat2cat.svg)](https://pypi.org/project/cat2cat/)\n\nUnifying an inconsistently coded categorical variable in a panel/longtitudal dataset.\n\n## Installation\n\n```bash\n$ pip install cat2cat\n```\n\n## Usage\n\nFor more examples and descriptions please vist [**the example notebook**](https://py-cat2cat.readthedocs.io/en/latest/example.html)\n\n### load example data\n\n```python\n# cat2cat datasets\nfrom cat2cat.datasets import load_trans, load_occup\ntrans = load_trans()\noccup = load_occup()\n```\n\n### Low-level functions\n\n```python\n# Low-level functions\nfrom cat2cat.mappings import get_mappings, get_freqs, cat_apply_freq\n\nmappings = get_mappings(trans)\ncodes_new = occup.code[occup.year == 2010].values\nfreqs = get_freqs(codes_new)\nmapp_new_p = cat_apply_freq(mappings["to_new"], freqs)\nmappings["to_new"][\'3481\']\nmapp_new_p[\'3481\']\n```\n\n### cat2cat function\n\n```python\nfrom cat2cat import cat2cat\nfrom cat2cat.dataclass import cat2cat_data, cat2cat_mappings, cat2cat_ml\n\nfrom pandas import DataFrame\n\no_old = occup.loc[occup.year == 2008, :].copy()\no_new = occup.loc[occup.year == 2010, :].copy()\n\n# dataclasses a core arguments for cat2cat function\ndata = cat2cat_data(\n    old = o_old, \n    new = o_new,\n    cat_var_old = "code", \n    cat_var_new = "code", \n    time_var = "year"\n)\nmappings = cat2cat_mappings(trans = trans, direction = "backward")\n\nc2c = cat2cat(data = data, mappings = mappings)\ndata_final = concat([c2c["old"], c2c["new"]])\n```\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`cat2cat` was created by Maciej Nasinski. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`cat2cat` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Maciej Nasinski',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/polkas/py-cat2cat',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.12',
}


setup(**setup_kwargs)
